// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition" by Clifford A. Shaffer, Prentice Hall, 2007.
// Source code Copyright (C) 2006 by Clifford A. Shaffer.

// Heapsort implementation and timing test driver
// This heapsort has moved all of the utility methods to be part of the
// main heap methods, so that there are far fewer function calls.

#include <iostream>
using namespace std;

#include "book.h"

// Include comparator functions
#include "compare.h"

// Optimized version of heapsort. All node access functions are
// replaced with direct computations on the array indices, so that there
// are far fewer function calls.
void heapsort(int A[], int n) { // Heapsort
  int mval; int pos;
  for (int i=n/2-1; i>=0; i--) {
    pos = i;
    while (pos < n/2) {     // Stop if pos is a leaf
      int j = 2*pos + 1;
      if (((j+1) < n) && (A[j] < A[j+1]))
        j++;        // Set j to greater child's value
      if (!(A[pos] < A[j])) break; // Done
      { int temp = A[pos]; A[pos] = A[j]; A[j] = temp; } // swap
      pos = j;         // Move down
    }
  }
  for (n--; n>0; n--) {     // Now sort
    // Swap max with last value
    { int temp = A[0]; A[0] = A[n]; A[n] = temp; } // swap
    pos = 0;
    while (pos < n/2) {     // Stop if pos is a leaf
      int j = 2*pos + 1;
      if (((j+1) < n) && (A[j] < A[j+1]))
        j++;        // Set j to greater child's value
      if (!(A[pos] < A[j])) break; // Done
      // swap
      { int temp = A[pos]; A[pos] = A[j]; A[j] = temp; }
      pos = j;         // Move down
    }
  }
}

template <class Elem, class Comp>
void sort(Elem* array, int n) {
 heapsort(array, n);
}

#include "sortmain.cpp"
