// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition" by Clifford A. Shaffer, Prentice Hall, 2007.
// Source code Copyright (C) 2006 by Clifford A. Shaffer.

// Test driver for syntax checking the various hashing routines

#include <iostream>
using namespace std;

#include "book.h"

// Include comparator functions
#include "compare.h"

#define HASHSIZE 101

int HT[HASHSIZE];
int M = HASHSIZE;

// Hash function for character strings.
// Use folding on a string, summed 4 bytes at a time
int sfold(char* key) {
  unsigned long *lkey = (unsigned long *)key;
  int length = strlen(key)/4;
  unsigned long sum = 0;
  for(int i=0; i<length; i++)
    sum += lkey[i];

  // Now deal with the chars at the end
  length = strlen(key) - length*4;
  char temp[4];
  lkey = (unsigned long *)temp;
  lkey[0] = 0;
  for(int i=0; i<length; i++)
    temp[i] = key[length*4+i];
  sum += lkey[0];
  return sum % M;
}

// Hash function for character strings:
// Simply sum up the ascii values per character
int h(char* x) {
  int i, sum;
  for (sum=0, i=0; x[i] != '\0'; i++)
    sum += (int) x[i];
  return(sum % M);
}

// Hash function for ints: take mod 16
int h(int x) {
  return(x % 16);
}

// Probe function example: linear probing
template <class Key>
int p(Key K, int i) { return i; }

int main(int argc, char** argv)
{
  if (argc == 2)
    cout << "Hashed |" << argv[1] << "| to " << sfold(argv[1]) << endl;

  return 0;
}
